ELEMENTS_CONFIG = {
    "terra": {
        "strongs": ["electric", "flame"],
        "weaknesses": ["metal", "war"]
    },
    "flame": {
        "strongs": ["nature", "ice"],
        "weaknesses": ["sea", "terra"]
    },
    "sea": {
        "strongs": ["flame", "war"],
        "weaknesses": ["nature", "electric"]
    },
    "nature": {
        "strongs": ["sea", "light"],
        "weaknesses": ["flame", "ice"]
    },
    "electric": {
        "strongs": ["sea", "metal"],
        "weaknesses": ["terra", "light"]
    },
    "ice": {
        "strongs": ["nature", "war"],
        "weaknesses": ["flame", "metal"]
    },
    "metal": {
        "strongs": ["terra", "ice"],
        "weaknesses": ["electric", "dark"]
    },
    "dark": {
        "strongs": ["metal", "light"],
        "weaknesses": ["war"]
    },
    "light": {
        "strongs": ["electric", "dark"],
        "weaknesses": ["nature"]
    },
    "war": {
        "strongs": ["terra", "dark"],
        "weaknesses": ["sea", "ice"]
    },
    "pure": {
        "strongs": ["wind"],
        "weaknesses": ["primal"]
    },
    "legend": {
        "strongs": ["primal"],
        "weaknesses": ["pure"]
    },
    "primal": {
        "strongs": ["pure"],
        "weaknesses": ["time"]
    },
    "wind": {
        "strongs": ["time"],
        "weaknesses": ["legend"]
    },
    "time": {
        "strongs": ["legend"],
        "weaknesses": ["wind"]
    },
    "happy": {
        "strongs": ["chaos", "magic"],
        "weaknesses": []
    },
    "chaos": {
        "strongs": ["magic", "soul"],
        "weaknesses": []
    },
    "magic": {
        "strongs": ["soul", "beauty"],
        "weaknesses": []
    },
    "soul": {
        "strongs": ["dream", "beauty"],
        "weaknesses": []
    },
    "beauty": {
        "strongs": ["dream", "happy"],
        "weaknesses": []
    },
    "dream": {
        "strongs": ["happy", "chaos"],
        "weaknesses": []
    }
}

__all__ = [ ELEMENTS_CONFIG ]


from pydantic import validate_arguments
from urllib.request import urlretrieve

class BaseStaticDownloader:
    url: str

    @validate_arguments
    def download(self, output: str):
        urlretrieve(self.url, output)

__all__ = [ BaseStaticDownloader ]


ORB_RECALL_CONFIG = {
    "per_levels": [ 40, 2, 2, 2, 2, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 2, 2, 2, 2, 2, 2, 2 ],
    "per_stars": [ 120, 200, 320, 560, 800 ]
}

DRAGON_MIN_LEVEL = 1
DRAGON_MAX_LEVEL = 70
DRAGON_MIN_STARS = 0
DRAGON_MAX_STARS = 5

__all__ = [
    ORB_RECALL_CONFIG,
    DRAGON_MIN_LEVEL,
    DRAGON_MAX_LEVEL
]


from pydantic import validate_arguments

from .config import ELEMENTS_CONFIG

@validate_arguments
def calculate_strongs(elements: list[str]) -> list[str]:
    strongs = []

    for element in elements:
        strong = ELEMENTS_CONFIG[element]["strongs"]

        for element in strong:
            if not element in strongs:
                strongs.append(element)

    return strongs

@validate_arguments
def calculate_weaknesses(first_element: str) -> list[str] | list:
    weaknesses = []

    for key in ELEMENTS_CONFIG.keys():
        if first_element in ELEMENTS_CONFIG[key]["strongs"]:
            weaknesses.append(key)

    return weaknesses

__all__ = [
    calculate_strongs,
    calculate_weaknesses
]


from pydantic import validate_arguments

from ...base import BaseStaticDownloader

class DragonFlashAnimation(BaseStaticDownloader):
    @validate_arguments
    def __init__(
        self,
        image_name: str,
        phase: int,
        skin: int | None = None
    ) -> None:
        if phase < 0 or phase > 3:
            raise ValueError(f"{phase} Not a valid number for a dragon's phase. Choose a number between 0 and 3")

        if skin:
            skin = f"_skin{skin}"
        
        else:
            skin = ""

        self.url = f"https://dci-static-s1.socialpointgames.com/static/dragoncity/assets/sprites/{image_name}{skin}_{phase}.swf"

__all__ = [ DragonFlashAnimation ]


from pydantic import validate_arguments

from ..base import BaseStaticDownloader

class DragonSprite(BaseStaticDownloader):
    @validate_arguments
    def __init__(
        self,
        image_name: str,
        phase: int,
        skin: int | None = None,
        image_quality: int = 1
    ) -> None:
        if phase < 0 or phase > 3:
            raise ValueError(f"{phase} Not a valid number for a dragon's phase. Choose a number between 0 and 3")

        if skin:
            skin = f"_skin{skin}"
        
        else:
            skin = ""

        if image_quality == 1:
            image_quality = ""

        elif image_quality == 2:
            image_quality = "@2x"

        else:
            raise ValueError(f"{image_quality} Not a valid number for image quality of a dragon. Choose a number between 1 and 2")

        self.url = f"https://dci-static-s1.socialpointgames.com/static/dragoncity/mobile/ui/dragons/ui_{image_name}{skin}_{phase}{image_quality}.png"

__all__ = [ DragonSprite ]


from ..base import BaseStaticDownloader

class IslandPackage(BaseStaticDownloader):
    def __init__(
        self,
        uri: str
    ) -> None:
        uri = uri.split("/")
        type_ = uri[3]
        filename = uri[4]

        if type_ == "grid_islands":
            filename.replace(".zip", "_optim.zip")

        self.url = f"https://www.socialpointgames.com/static/dragoncity/mobile/ui/{type_}/HD/dxt5/{filename}"

__all__ = [ IslandPackage ]


from pydantic import validate_arguments

from ...base import BaseStaticDownloader

WINDOWS = "dxt5"

class DragonSpineAnimation(BaseStaticDownloader):
    @validate_arguments
    def __init__(
        self,
        image_name: str,
        phase: int,
        skin: int | None = None,
        platform: str = WINDOWS,
        use_new_url: bool = True
    ) -> None:
        if phase < 0 or phase > 3:
            raise ValueError(f"{phase} Not a valid number for a dragon's phase. Choose a number between 0 and 3")

        if skin:
            skin = f"_skin{skin}"
        
        else:
            skin = ""

        if use_new_url:
            self.url = f"https://dci-static-s1.socialpointgames.com/static/dragoncity/mobile/engine/version_1_1/dragons/{image_name}_{phase}/{image_name}_{phase}_HD_tweened_{platform}.zip"
            
        else:
            self.url = f"https://dci-static-s1.socialpointgames.com/static/dragoncity/mobile/engine/version_1_1/dragons/{image_name}_{phase}/basic_{image_name}_{phase}_HD_spine-3-8-59_{platform}.zip"

__all__ = [ DragonSpineAnimation ]


from .elements import calculate_strongs, calculate_weaknesses
from .orb_recall import calculate_orb_recall_gain

__all__ = [
    calculate_strongs,
    calculate_weaknesses,
    calculate_orb_recall_gain
]


from ..base import BaseStaticDownloader

class GeneralMusic(BaseStaticDownloader):
    def __init__(
        self,
        music_name: str
    ) -> None:
        self.url = f"http://dci-static-s1.socialpointgames.com/static/dragoncity/mobile/sounds/music/{music_name}.mp3"

__all__ = [ GeneralMusic ]


from pydantic import validate_arguments

from .config import (
    ORB_RECALL_CONFIG,
    DRAGON_MIN_LEVEL,
    DRAGON_MAX_LEVEL,
    DRAGON_MIN_STARS,
    DRAGON_MAX_STARS
)

@validate_arguments
def calculate_orb_recall_gain(dragon_level: int, dragon_stars: int) -> int:
    orb_recall_gain = 0

    if dragon_level < DRAGON_MIN_LEVEL or dragon_level > DRAGON_MAX_LEVEL:
        raise ValueError(f"'{dragon_level}' is not a valid level for a dragon, choose a level between '{DRAGON_MIN_LEVEL}' and '{DRAGON_MAX_LEVEL}'")

    if dragon_stars < DRAGON_MIN_STARS or dragon_stars > DRAGON_MAX_STARS:
        raise ValueError(f"'{dragon_level}' It is not a number of stars valid for a dragon, choose a number of stars between '{DRAGON_MIN_STARS}' and '{DRAGON_MAX_STARS}'")

    for i in range(dragon_level if dragon_level <= 30 else 30):
        orb_recall_gain += ORB_RECALL_CONFIG["per_levels"][i]

    for i in range(dragon_stars):
        orb_recall_gain += ORB_RECALL_CONFIG["per_stars"][i]

    return orb_recall_gain

__all__ = [ calculate_orb_recall_gain ]


from pydantic import validate_arguments
from pyfilter import FromList
import httpx

class Localization:
    @validate_arguments
    def __init__(self, language: str) -> None:
        self.__endpoint_url = f"https://sp-translations.socialpointgames.com/deploy/dc/android/prod/dc_android_{language}_prod_wetd46pWuR8J5CmS.json"
        self.__localization = self.__fetch()
        self.__localization_dict = FromList(self.__localization).join_keys_of_child_dicts_in_a_new_dict()

    def __fetch(self) -> list[dict]:
        response = httpx.get(self.__endpoint_url)
        data = response.json()
        return data

    @classmethod
    def load(cls, loc: list[dict]):
        cls.__localization = loc
        cls.__localization_dict = FromList(loc).join_keys_of_child_dicts_in_a_new_dict()

        return cls

    @validate_arguments
    @classmethod
    def get_value_from_key(cls, key: str) -> str | None:
        if key in cls.__localization_dict.keys():
            return cls.__localization_dict[key]

    @validate_arguments
    @classmethod
    def get_key_from_value(cls, value: str) -> str | None:
        for dict_key, dict_value in cls.__localization_dict.items():
            if dict_value == value:
                return dict_key

    @validate_arguments
    @classmethod
    def get_dragon_name(cls, id: int) -> str | None:
        key = f"tid_unit_{id}_name"
        return cls.get_value_from_key(key)

    @validate_arguments
    @classmethod
    def get_dragon_description(cls, id: int) -> str | None:
        key = f"tid_unit_{id}_description"
        return cls.get_value_from_key(key)

    @validate_arguments
    @classmethod
    def get_attack_name(cls, id: int) -> str | None:
        key = f"tid_attack_name_{id}"
        return cls.get_value_from_key(key)

    @validate_arguments
    @classmethod
    def get_skill_name(cls, id: int) -> str | None:
        key = f"tid_skill_name_{id}"
        return cls.get_value_from_key(key)

    @validate_arguments
    @classmethod
    def get_skill_description(cls, id: int) -> str | None:
        key = f"tid_skill_description_{id}"
        return cls.get_value_from_key(key)

    @validate_arguments
    @classmethod
    def search_keys(cls, query: str) -> list[str] | list:
        query = (query
            .lower()
            .strip())

        results = []

        for key in cls.__localization_dict.keys():
            parsed_key = (key
                .lower()
                .strip())

            if query in parsed_key:
                results.append(key)

        return results

    @validate_arguments
    @classmethod
    def search_values(cls, query: str) -> list[str] | list:
        query = (query
            .lower()
            .strip())

        results = []

        for value in cls.__localization_dict.values():
            parsed_value = (value
                .lower()
                .strip())

            if query in parsed_value:
                results.append(value)

        return results

    @validate_arguments
    @classmethod
    def compare(cls, old_localization: dict | list) -> dict[str, list]:
        if type(old_localization) == list:
            old_localization = FromList(old_localization).join_keys_of_child_dicts_in_a_new_dict()

        new_fields = []
        edited_fields = []

        old_localization_keys = old_localization.keys()
        for key in cls.__localization_dict.keys():
            if key not in old_localization_keys:
                new_fields.append({
                    "key": key,
                    "value": cls.__localization_dict[key]
                })

        for key in old_localization_keys:
            if old_localization[key] != cls.__localization_dict[key]:
                edited_fields.append({
                    "key": key,
                    "old_value": old_localization[key],
                    "new_value": cls.__localization_dict[key]
                })

        return dict(
            new_fields = new_fields,
            edited_fields = edited_fields
        )

    @classmethod
    def get(cls):
        return cls.__localization

    @classmethod
    def get_dict(cls):
        return cls.__localization_dict

__all__ = [ Localization ]


from setuptools import setup, find_packages

with open("README.md", "r") as file:
    readme_content = file.read()

setup(
    name="dragon-city-utils",
    version="1.8",
    license="MIT License",
    author="Marcuth",
    long_description=readme_content,
    long_description_content_type="text/markdown",
    author_email="marcuth2006@gmail.com",
    keywords="dragoncity dcutils tools",
    description=f"Utilities and tools for things related to Dragon City",
    packages=[ "dcutils/" + x for x in find_packages("dcutils") ],
    install_requires=["httpx", "pydantic", "python-filter"]
)


from pydantic import validate_arguments

from ..base import BaseStaticDownloader

class ChestSprite(BaseStaticDownloader):
    @validate_arguments
    def __init__(
        self,
        image_name: str,
        image_quality: int = 1
    ) -> None:
        if image_quality == 1:
            image_quality = ""

        elif image_quality == 2:
            image_quality = "@2x"
        
        self.url = f"https://dci-static-s1.socialpointgames.com/static/dragoncity/mobile/ui/chests/ui_{image_name}{image_quality}.png"

__all__ = [ ChestSprite ]